!==============================================================================!
!==============================================================================!
!                                                                              |
!              ||===\\                                                         | 
!              ||    \\                                                        |
!              ||     ||   //==\\   ||  ||   //==||  ||/==\\                   |
!              ||     ||  ||    ||  ||  ||  ||   ||  ||    ||                  |
!              ||    //   ||    ||  ||  ||  ||   ||  ||                        |
!              ||===//     \\==//    \\==\\  \\==\\  ||                        |
!                                                                              |
!==============================================================================!
!==============================================================================!
!                                                                              |
!              TRACKING CLOUD POST PROCESSING   Jan. 2015                      |
!                                                                              |
!==============================================================================!
!==============================================================================!

program convert_trfile

implicit none

!buffer variables for reading
integer np,ntracks,istep
real,dimension(:),allocatable :: x,y,z,press,temp
integer,dimension(:,:),allocatable :: tracks
real :: time,lref,veloref,rhoref,tempscale

!other variables
integer :: i,err,outmode
character(150) :: infile,outfile

write (*,'(a)') '!------------------------------------------------------------------------------|'
write (*,'(a)') '!------------------------------------------------------------------------------|'
write (*,'(a)') '!                                                                              |'
write (*,'(a)') '!          8888888b.   .d88888b.  888     888       d8888 8888888b.            |'
write (*,'(a)') '!          888  "Y88b d88P" "Y88b 888     888      d88888 888   Y88b           |'
write (*,'(a)') '!          888    888 888     888 888     888     d88P888 888    888           |'
write (*,'(a)') '!          888    888 888     888 888     888    d88P 888 888   d88P           |'
write (*,'(a)') '!          888    888 888     888 888     888   d88P  888 8888888P"            |'
write (*,'(a)') '!          888    888 888     888 888     888  d88P   888 888 T88b             |'
write (*,'(a)') '!          888  .d88P Y88b. .d88P Y88b. .d88P d8888888888 888  T88b            |'
write (*,'(a)') '!          8888888P"   "Y88888P"   "Y88888P" d88P     888 888   T88b           |'
write (*,'(a)') '!                                                                              |'
write (*,'(a)') '!            DOUAR-WSMP bin2ascii-converter for the tracking cloud             |'
write (*,'(a)') '!------------------------------------------------------------------------------|'
write (*,'(a)') '!------------------------------------------------------------------------------|'
write (*,*) ''

!------------------------------------------------------------------------------|
!    Input parameters
!------------------------------------------------------------------------------|

write (*,'(a)') 'Please enter the input file:'
read (*,'(a)') infile
write (*,'(a)') 'Please enter the output mode (1=screen,2=file,3=both):'
read (*,'(i)') outmode
if (outmode>1) then
    write (*,'(a)') 'Please enter the output file:'
    read (*,'(a)') outfile
end if

!------------------------------------------------------------------------------|
!    Reading Tracking Cloud Files
!------------------------------------------------------------------------------|

!reading input file header for array sizes

open (unit=20,file=trim(infile),status='old',form='unformatted')
read (20) np,ntracks,istep,time
read (20) lref,veloref,rhoref,tempscale

!allocating master arrays for input
allocate (x(np))
allocate (y(np))
allocate (z(np))
allocate (temp(np))
allocate (press(np))
allocate (tracks(ntracks,3))

!read all information for this time step
read (20) x
read (20) y
read (20) z
read (20) temp
read (20) press
do i=1,ntracks
    read (20) tracks(i,:)
end do
close (20)

if (outmode==1 .or. outmode==3) then
    write (*,*) 'np, ntracks, istep, time:'
    write (*,*) np,ntracks,istep,time
    write (*,*) 'lref, veloref, rhoref, tempscale: '
    write (*,*) lref,veloref,rhoref,tempscale
    write (*,*) 'Particle positions: x, y, z:'
    do i=1,np
        write (*,*) x(i), y(i), z(i)
    end do
    write (*,*) 'Particle fields: pressure, temperature:'
    do i=1,np
        write (*,*) press(i), temp(i)
    end do
    write (*,*) 'Surfaced tracks: ID, startstep, endstep:'
    do i=1,ntracks
        write (*,*) tracks(i,:)
    end do
end if

if (outmode==2 .or. outmode==3) then
    open(unit=30,file=trim(outfile),status='replace')
    write (30,*) 'np, ntracks, istep, time:'
    write (30,*) np,ntracks,istep,time
    write (30,*) 'lref, veloref, rhoref, tempscale: '
    write (30,*) lref,veloref,rhoref,tempscale
    write (30,*) 'Particle positions: x, y, z:'
    do i=1,np
        write (30,*) x(i), y(i), z(i)
    end do
    write (30,*) 'Particle fields: pressure, temperature:'
    do i=1,np
        write (30,*) press(i), temp(i)
    end do
    write (30,*) 'Surfaced tracks: ID, startstep, endstep:'
    do i=1,ntracks
        write (30,*) tracks(i,:)
    end do
    close (30)
end if

deallocate (x)
deallocate (y)
deallocate (z)
deallocate (temp)
deallocate (press)
deallocate (tracks)

end
