subroutine write_post_track (trcl,surface,inparam,outparam)

use trdefinitions
use m_toolbox

implicit none

include 'mpif.h'

type (tracking_cloud) :: trcl
type (input_parameters) :: inparam
type (output_parameters) :: outparam
type (trsheet), dimension(outparam%nstep) :: surface

integer :: write_nstep,write_points,write_tracks
integer :: i,k,l,l2,ipoint

real :: maxdepth,maxtemp,maxpress

character(4) :: cistep

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

if (btest(outparam%mode,0) .and. trcl%ntracks>0) then
!output age points and interpolated age surface for each output timestep
writeages: do l=outparam%first,outparam%last,outparam%step
    !l2 is the index for step l in master array
    l2 = (l-outparam%first)/outparam%step+1

    !count number of tracks for this timestep
    write_tracks = 0
    do i=1,trcl%ntracks
        if (trcl%tracks(i,5)==l) then
            write_tracks = write_tracks+1
        end if
    end do

    !create output file age calculation results
    call int_to_char (cistep,4,l)
    open (unit=40,file=trim(outparam%dir)//'ages-points_t'//cistep//'.vtk',status='replace')

    !write header
    write(40,'(a)') '# vtk DataFile Version 3.1'
    write(40, '(a,i5,a,i4,a)') 'DOUAR thermochron ages for timestep',l, &
        ', accumulating',outparam%keep_nstep,' timesteps'
    write(40, '(a)') 'ASCII'
    write(40, '(a)') 'DATASET UNSTRUCTURED_GRID'
    write(40, '(a)') ''
    write(40,'(a)')'FIELD FieldData 2'
    if (outparam%scaled==1) then
        write(40,'(a)')'Time_Ma 1 1 double'
    else
        write(40,'(a)')'Time_DOUAR 1 1 double'
    end if
    write(40,'(ES12.5E2)') trcl%time(l2)
    write(40,'(a)')'Step 1 1 int'
    write(40,'(i6)') l
    write(40, '(a)') ''

    anytracks1: if (write_tracks==0) then

    write (*,'(a,i4)') 'No surfaced tracks for step ', l

    write(40, '(a,i10,a)') 'POINTS 1 FLOAT'
    write(40,'(3ES12.5E2)') 0.d0, 0.d0, 0.d0 
    write(40, '(a)') '' 
    write(40, '(a)') 'CELLS 1 2'
    write(40, '(i1,i6)') 1, 0
    write(40, '(a)') '' 
    !write cell data type, 1=single vertex
    write(40, '(a)') 'CELL_TYPES 1'
    write(40, '(i2)') 1

    else anytracks1
    !write point coordinates
    write(40, '(a,i10,a)') 'POINTS ', write_tracks, ' FLOAT'
    do i=1,trcl%ntracks
    !go through all tracks
        if (trcl%tracks(i,5)==l) then
        !if track surfaces at this timestep: write last point of track (at surface)
        write(40,'(3ES12.5E2)') trcl%x(trcl%tracks(i,1),trcl%tracks(i,3)), &
            trcl%y(trcl%tracks(i,1),trcl%tracks(i,3)), & 
            trcl%z(trcl%tracks(i,1),trcl%tracks(i,3))
        end if
    end do
    write(40, '(a)') ''

    !write cell data
    write(40, '(a,i10,i10)') 'CELLS ', write_tracks, 2*write_tracks
    ipoint = 0
    do i=1,trcl%ntracks
        if (trcl%tracks(i,5)==l) then
        !write length of track
        write(40, '(i1,i6)') 1, ipoint
        !advance point ID
        ipoint = ipoint+1
        end if
    end do
    write (40, '(a)') '' 
    

    !write cell data type, 1=single vertex
    write(40, '(a,i10)') 'CELL_TYPES ',write_tracks
    do i = 1,write_tracks
        write(40, '(i2)') 1
    end do
    write (40, '(a)') '' 

    !write point datasets
    if (outparam%nages > 0) then
        write (40,'(a,i10)') 'POINT_DATA ',write_tracks
        
        if (outparam%Mad_AHe_std ==1) then
            write (40,'(a)') 'SCALARS Mad_AHe_std FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_AHe_std(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_Ahe_small ==1) then
            write (40,'(a)') 'SCALARS Mad_Ahe_small FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_Ahe_small(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_Ahe_medium ==1) then
            write (40,'(a)') 'SCALARS Mad_Ahe_medium FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_Ahe_medium(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_Ahe_large ==1) then
            write (40,'(a)') 'SCALARS Mad_Ahe_large FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_Ahe_large(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_Ahe_lowdam ==1) then
            write (40,'(a)') 'SCALARS Mad_Ahe_LowDamage FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_Ahe_lowdam(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_Ahe_moddam ==1) then
            write (40,'(a)') 'SCALARS Mad_Ahe_ModerateDamage FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_Ahe_moddam(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_Ahe_hidam ==1) then
            write (40,'(a)') 'SCALARS Mad_Ahe_HighDamage FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_Ahe_hidam(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_ZHe ==1) then
            write (40,'(a)') 'SCALARS Mad_ZHe FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_ZHe(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_ArKFs ==1) then
            write (40,'(a)') 'SCALARS Mad_Ar-KFeldspat FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_ArKFs(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_ArBio ==1) then
            write (40,'(a)') 'SCALARS Mad_ArBiotite FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_ArBio(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_ArHorn ==1) then
            write (40,'(a)') 'SCALARS Mad_ArHornblende FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_ArHorn(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_ArMusc ==1) then
            write (40,'(a)') 'SCALARS Mad_ArMuscovite FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_ArMusc(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_Ti_UTh_He ==1) then
            write (40,'(a)') 'SCALARS Mad_Ti_UTh_He FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_Ti_UTh_He(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_AFT ==1) then
            write (40,'(a)') 'SCALARS Mad_AFT FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_AFT(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Mad_ZFT ==1) then
            write (40,'(a)') 'SCALARS Mad_ZFT FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Mad_ZFT(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Muscovite ==1) then
            write (40,'(a)') 'SCALARS Muscovite FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Muscovite(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%ZFT ==1) then
            write (40,'(a)') 'SCALARS ZFT FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%ZFT(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Apatite_U_Th_Pb ==1) then
            write (40,'(a)') 'SCALARS Apatite_U_Th_Pb FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Apatite_U_Th_Pb(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Biotite ==1) then
            write (40,'(a)') 'SCALARS Biotite FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Biotite(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Rutile_U_Pb ==1) then
            write (40,'(a)') 'SCALARS Rutile_U_Pb FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Rutile_U_Pb(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Titanite_U_Pb ==1) then
            write (40,'(a)') 'SCALARS Titanite_U_Pb FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Titanite_U_Pb(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%Zircon_U_Pb ==1) then
            write (40,'(a)') 'SCALARS Zircon_U_Pb FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(3ES12.5E2)') trcl%Zircon_U_Pb(i)
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%origin==1) then
            write (40,'(a)') 'VECTORS track_origin FLOAT'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                !write first point of track 
                write(40,'(3ES12.5E2)') trcl%x(trcl%tracks(i,1),trcl%tracks(i,2)), &
                    trcl%y(trcl%tracks(i,1),trcl%tracks(i,2)), & 
                    trcl%z(trcl%tracks(i,1),trcl%tracks(i,2))
            end if
            end do
            write(40, '(a)') ''
        end if
        if (outparam%maxdepth==1) then
            write (40,'(a)') 'SCALARS max_depth FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                !Get minimum Z value of track
                maxdepth = minval(trcl%z(trcl%tracks(i,1), trcl%tracks(i,2):trcl%tracks(i,3)))
                write(40,'(ES12.5E2)') maxdepth
            end if
            end do
            write(40, '(a)') ''
        end if
        if (outparam%maxtemp==1) then
            write (40,'(a)') 'SCALARS max_temperature FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                !Get maximum temperature value of track
                maxtemp = maxval(trcl%temp(trcl%tracks(i,1), trcl%tracks(i,2):trcl%tracks(i,3)))
                write(40,'(ES12.5E2)') maxtemp
            end if
            end do
            write(40, '(a)') ''
        end if
        if (outparam%maxpressure==1) then
            write (40,'(a)') 'SCALARS max_pressure FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                !Get maximum pressure value of track
                maxpress = abs(minval(trcl%press(trcl%tracks(i,1), trcl%tracks(i,2):trcl%tracks(i,3))))
                write(40,'(ES13.5E2)') maxpress
            end if
            end do
            write(40, '(a)') ''
        end if

        if (outparam%ageID==1) then
            write (40,'(a)') 'SCALARS trackID FLOAT 1'
            write (40,'(a)') 'LOOKUP_TABLE default'
            do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                write(40,'(i5)') trcl%tracks(i,7)
            end if
            end do
            write(40, '(a)') ''
        end if
    end if
    
    end if anytracks1
    close (40) 
    
    !write suface with interpolated ages
    open (unit=41,file=trim(outparam%dir)//'ages-surface_t'//cistep//'.vtk',status='replace')
    write(41,'(a)') '# vtk DataFile Version 3.1'
    write(41, '(a,i5,a,i4,a)') 'DOUAR thermochron ages for timestep',l, &
        ', accumulating',outparam%keep_nstep,' timesteps'
    write(41, '(a)') 'ASCII'
    write(41, '(a)') 'DATASET UNSTRUCTURED_GRID'
    write(41, '(a)') ''
    write(41,'(a)')'FIELD FieldData 2'
    if (outparam%scaled==1) then
        write(41,'(a)')'Time_Ma 1 1 double'
    else
        write(41,'(a)')'Time_DOUAR 1 1 double'
    end if
    write(41,'(ES12.5E2)') trcl%time(l2)
    write(41,'(a)')'Step 1 1 int'
    write(41,'(i6)') l
    write(41, '(a)') ''

    anytracks2: if (write_tracks < 4) then

    write(41, '(a,i10,a)') 'POINTS 1 FLOAT'
    write(41,'(3ES12.5E2)') 0.d0, 0.d0, 0.d0 
    write(41, '(a)') '' 
    write(41, '(a)') 'CELLS 1 2'
    write(41, '(i1,i6)') 1, 0
    write(41, '(a)') '' 
    write(41, '(a)') 'CELL_TYPES 1'
    write(41, '(i2)') 1

    else anytracks2

    write(41,'(a7,i10,a6)')'POINTS ',surface(l2)%nsurface,' float'
    do i=1,surface(l2)%nsurface
      write(41,'(3ES12.5E2)')surface(l2)%x(i),surface(l2)%y(i),surface(l2)%z(i)
    enddo
    write(41,'(a6, 2I10)') 'CELLS ',surface(l2)%nt,(1+3)*surface(l2)%nt
    do i=1,surface(l2)%nt
      write(41,'(9I10)')3,surface(l2)%icon(1:3,i)-1
    enddo
    write(41,'(A11, I12)') 'CELL_TYPES ',surface(l2)%nt
    do i=1,surface(l2)%nt
      write(41,'(I2)') 5 ! triangle  (3 nodes)
    end do

    if (outparam%nages > 0) then
        k=1
        write (41,'(a,i10)') 'POINT_DATA ', surface(l2)%nsurface
        
        if (outparam%Mad_AHe_std ==1) then
            write (41,'(a)') 'SCALARS Mad_AHe_std FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_Ahe_small ==1) then
            write (41,'(a)') 'SCALARS Mad_Ahe_small FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_Ahe_medium ==1) then
            write (41,'(a)') 'SCALARS Mad_Ahe_medium FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_Ahe_large ==1) then
            write (41,'(a)') 'SCALARS Mad_Ahe_large FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_Ahe_lowdam ==1) then
            write (41,'(a)') 'SCALARS Mad_Ahe_LowDamage FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_Ahe_moddam ==1) then
            write (41,'(a)') 'SCALARS Mad_Ahe_ModerateDamage FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_Ahe_hidam ==1) then
            write (41,'(a)') 'SCALARS Mad_Ahe_HighDamage FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_ZHe ==1) then
            write (41,'(a)') 'SCALARS Mad_ZHe FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_ArKFs ==1) then
            write (41,'(a)') 'SCALARS Mad_Ar-KFeldspat FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_ArBio ==1) then
            write (41,'(a)') 'SCALARS Mad_ArBiotite FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_ArHorn ==1) then
            write (41,'(a)') 'SCALARS Mad_ArHornblende FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_ArMusc ==1) then
            write (41,'(a)') 'SCALARS Mad_ArMuscovite FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_Ti_UTh_He ==1) then
            write (41,'(a)') 'SCALARS Mad_Ti_UTh_He FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_AFT ==1) then
            write (41,'(a)') 'SCALARS Mad_AFT FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Mad_ZFT ==1) then
            write (41,'(a)') 'SCALARS Mad_ZFT FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Muscovite ==1) then
            write (41,'(a)') 'SCALARS Muscovite FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%ZFT ==1) then
            write (41,'(a)') 'SCALARS ZFT FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Apatite_U_Th_Pb ==1) then
            write (41,'(a)') 'SCALARS Apatite_U_Th_Pb FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Biotite ==1) then
            write (41,'(a)') 'SCALARS Biotite FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Rutile_U_Pb ==1) then
            write (41,'(a)') 'SCALARS Rutile_U_Pb FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Titanite_U_Pb ==1) then
            write (41,'(a)') 'SCALARS Titanite_U_Pb FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%Zircon_U_Pb ==1) then
            write (41,'(a)') 'SCALARS Zircon_U_Pb FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%origin ==1) then
            write (41,'(a)') 'VECTORS track_origin FLOAT'
            do i=1,surface(l2)%nsurface
                write(41,'(3ES12.5E2)') surface(l2)%ages(i,k:k+2)
            end do
            write(41, '(a)') ''
            k = k+3
        end if
        
        if (outparam%maxdepth ==1) then
            write (41,'(a)') 'SCALARS max_depth FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if

        if (outparam%maxtemp ==1) then
            write (41,'(a)') 'SCALARS max_temperature FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if
        if (outparam%maxpressure ==1) then
            write (41,'(a)') 'SCALARS max_pressure FLOAT 1'
            write (41,'(a)') 'LOOKUP_TABLE default'
            do i=1,surface(l2)%nsurface
                write(41,'(ES12.5E2)') surface(l2)%ages(i,k)
            end do
            write(41, '(a)') ''
            k = k+1
        end if
    end if

    end if anytracks2
    close(41)
end do writeages
end if

outparam%ntrackvars = outparam%velo+outparam%veloz+outparam%temp+outparam%press+ &
    outparam%time+outparam%istep+outparam%trackID

if (btest(outparam%mode,1) .and. trcl%ntracks>0) then
!output surfaced tracks for each output timestep
    do l=outparam%first,outparam%last,outparam%step
    !count number of tracks and number of points in those for this timestep
    write_tracks = 0
    write_points = 0
    do i=1,trcl%ntracks
        if (trcl%tracks(i,5)==l) then
        write_tracks = write_tracks+1
        write_points = write_points+trcl%tracks(i,4)
        end if
    end do

    call int_to_char (cistep,4,l)
    open (unit=30,file=trim(outparam%dir)//'tracks_t'//cistep//'.vtk',status='replace')

    !write header
    write(30,'(a)') '# vtk DataFile Version 3.1'
    write(30, '(a)') 'DOUAR tracking cloud output of surfaced tracks'
    write(30, '(a)') 'ASCII'
    write(30, '(a)') 'DATASET UNSTRUCTURED_GRID'
    write(30, '(a)') ''
    write(30,'(a)')'FIELD FieldData 2'
    if (outparam%scaled==1) then
        write(30,'(a)')'Time_Ma 1 1 double'
    else
        write(30,'(a)')'Time_DOUAR 1 1 double'
    end if
    !l2 is the index for step l in master array
    l2 = (l-outparam%first)/outparam%step+1
    write(30,'(ES12.5E2)') trcl%time(l2)
    write(30,'(a)')'Step 1 1 int'
    write(30,'(i6)') l
    write(30, '(a)') ''

    if (write_tracks==0) then

        if (.NOT. btest(outparam%mode,0)) write (*,'(a,i4)') 'No surfaced tracks for step ', l

        write(30, '(a,i10,a)') 'POINTS 1 FLOAT'
        write(30,'(3ES12.5E2)') 0.d0, 0.d0, 0.d0
    else

        !write point coordinates
        write(30, '(a,i10,a)') 'POINTS ', write_points, ' FLOAT'
        do i=1,trcl%ntracks
        !go through all tracks
        if (trcl%tracks(i,5)==l) then
        !if track surfaces at this timestep:
            do k=trcl%tracks(i,2),trcl%tracks(i,3)
            !write all point positions within track extent
            write(30,'(3ES12.5E2)') trcl%x(trcl%tracks(i,1),k), &
                trcl%y(trcl%tracks(i,1),k),trcl%z(trcl%tracks(i,1),k)
            end do
        end if
        end do
        write(30, '(a)') ''

        !write cell data
        write(30, '(a,i10,i10)') 'CELLS ', write_tracks, write_points+write_tracks
        ipoint = 0
        do i=1,trcl%ntracks
        if (trcl%tracks(i,5)==l) then
            !write length of track
            write(30, '(i4)',advance='no') trcl%tracks(i,4)
            do k=trcl%tracks(i,2),trcl%tracks(i,3)
            !write point ID
            write(30, '(i9)', advance='no') ipoint
            !advance point ID
            ipoint = ipoint+1
            end do
            !end of track
            write (30, '(a)') '' 
        end if
        end do
        write (30, '(a)') '' 
        

        !write cell data type, 4=polyline
        write(30, '(a,i10)') 'CELL_TYPES ',write_tracks
        do i = 1,write_tracks
        write(30, '(i2)') 4
        end do
        write (30, '(a)') '' 

        !write point datasets
        if (outparam%ntrackvars > 0) write (30,'(a,i10)') 'POINT_DATA ',write_points
        
        if (outparam%velo==1) then
        write (30,'(a)') 'SCALARS Velo FLOAT 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
            do k=trcl%tracks(i,2),trcl%tracks(i,3)
                write(30,'(3ES12.5E2)') trcl%velo(trcl%tracks(i,1),k)
            end do
            end if
        end do
        write(30, '(a)') ''
        end if

        if (outparam%veloz==1) then
        write (30,'(a)') 'SCALARS VeloZ FLOAT 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
            do k=trcl%tracks(i,2),trcl%tracks(i,3)
                write(30,'(3ES12.5E2)') trcl%veloz(trcl%tracks(i,1),k)
            end do
            end if
        end do
        write(30, '(a)') ''
        end if

        if (outparam%temp==1) then
        write (30,'(a)') 'SCALARS Temperature FLOAT 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
            do k=trcl%tracks(i,2),trcl%tracks(i,3)
                write(30,'(3ES12.5E2)') trcl%temp(trcl%tracks(i,1),k)
            end do
            end if
        end do
        write(30, '(a)') ''
        end if

        if (outparam%press==1) then
        write (30,'(a)') 'SCALARS Pressure FLOAT 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
            do k=trcl%tracks(i,2),trcl%tracks(i,3)
                write(30,'(3ES12.5E2)') trcl%press(trcl%tracks(i,1),k)
            end do
            end if
        end do
        write(30, '(a)') ''
        end if

        if (outparam%time==1) then
        write (30,'(a)') 'SCALARS Time FLOAT 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
            do k=trcl%tracks(i,2),trcl%tracks(i,3)
                write(30,'(3ES12.5E2)') trcl%time(k)
            end do
            end if
        end do
        write(30, '(a)') ''
        end if

        if (outparam%istep==1) then
        write (30,'(a)') 'SCALARS Step INTEGER 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
            do k=trcl%tracks(i,2),trcl%tracks(i,3)
                write(30,'(i5)') trcl%step(k)
            end do
            end if
        end do
        write(30, '(a)') ''
        end if

        if (outparam%trackID==1) then
        write (30,'(a)') 'SCALARS Track_ID INTEGER 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
            do k=trcl%tracks(i,2),trcl%tracks(i,3)
                write(30,'(i5)') trcl%tracks(i,7)
            end do
            end if
        end do
        write(30, '(a)') ''
        end if

    end if
    close (30)

    end do
end if

if (btest(outparam%mode,2)) then
!output all tracks
    do l=outparam%first,outparam%last,outparam%step
    call int_to_char (cistep,4,l)
    open (unit=30,file=trim(outparam%dir)//'alltracks_t'//cistep//'.vtk',status='replace')

    !calculate part of array to write
    write_nstep = (l-inparam%first)/inparam%step+1

    !write header
    write(30,'(a)') '# vtk DataFile Version 3.1'
    write(30, '(a)') 'DOUAR tracking cloud output of all tracks'
    write(30, '(a)') 'ASCII'
    write(30, '(a)') 'DATASET UNSTRUCTURED_GRID'
    write(30, '(a)') ''

    !write point coordinates
    write(30, '(a,i8,a)') 'POINTS ', trcl%np*write_nstep, ' FLOAT'
    do i=1,trcl%np
        do k=1,write_nstep
        write(30,'(3ES12.5E2)') trcl%x(i,k),trcl%y(i,k),trcl%z(i,k)
        end do
    end do
    write(30, '(a)') ''

    !write cell data
    write(30, '(a,i10,i10)') 'CELLS ',trcl%np, trcl%np*(write_nstep+1)
    do i = 0,trcl%np-1
        write(30, '(i4)',advance='no') write_nstep
        do k = 1,write_nstep
        write(30, '(i8)', advance='no') i*write_nstep+k-1
        end do
        write (30, '(a)') '' 
    end do
    write (30, '(a)') '' 

    !write cell data type, 4=polyline
    write(30, '(a,i10)') 'CELL_TYPES ',trcl%np
    do i = 1,trcl%np
        write(30, '(i2)') 2
    end do
    write (30, '(a)') '' 

    !write point datasets
    if (outparam%ntrackvars > 0) write (30,'(a,i10)') 'POINT_DATA ',trcl%np*write_nstep
    
    if (outparam%velo==1) then
        write (30,'(a)') 'SCALARS Velo FLOAT 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%np
        do k=1,write_nstep
            write(30,'(ES12.5E2)') trcl%velo(i,k)
        end do
        end do
        write(30, '(a)') ''
    end if

    if (outparam%veloz==1) then
        write (30,'(a)') 'SCALARS VeloZ FLOAT 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%np
        do k=1,write_nstep
            write(30,'(ES12.5E2)') trcl%veloz(i,k)
        end do
        end do
        write(30, '(a)') ''
    end if

    if (outparam%temp==1) then
        write (30,'(a)') 'SCALARS Temperature FLOAT 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%np
        do k=1,write_nstep
            write(30,'(ES12.5E2)') trcl%temp(i,k)
        end do
        end do
        write(30, '(a)') ''
    end if

    if (outparam%press==1) then
        write (30,'(a)') 'SCALARS Pressure FLOAT 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%np
        do k=1,write_nstep
            write(30,'(ES12.5E2)') trcl%press(i,k)
        end do
        end do
        write(30, '(a)') ''
    end if

    if (outparam%time==1) then
        write (30,'(a)') 'SCALARS Time FLOAT 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%np
        do k=1,write_nstep
            write(30,'(ES12.5E2)') trcl%time(k)
        end do
        end do
        write(30, '(a)') ''
    end if

    if (outparam%istep==1) then
        write (30,'(a)') 'SCALARS Step INTEGER 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%np
        do k=1,write_nstep
            write(30,'(i5)') trcl%step(k)
        end do
        end do
        write(30, '(a)') ''
    end if

    if (outparam%trackID==1) then
        write (30,'(a)') 'SCALARS Particle_ID INTEGER 1'
        write (30,'(a)') 'LOOKUP_TABLE default'
        do i=1,trcl%np
        do k=1,write_nstep
            write(30,'(i6)') i
        end do
        end do
        write(30, '(a)') ''
    end if

    close (30)
    end do
end if

end subroutine write_post_track
