module m_write_tecplot 
contains

subroutine write_post_track_tecplot (trcl,surface,outparam)

use trdefinitions
use m_toolbox

implicit none

include 'mpif.h'

type (tracking_cloud) :: trcl
type (output_parameters) :: outparam
type (trsheet), dimension(outparam%nstep) :: surface

integer :: write_tracks
integer :: i, j, k, l, l2, charcnt, nvar, ivar

real :: maxdepth

character(4) :: cistep

!------------------------------------------------------------------------------|
!------------------------------------------------------------------------------|

if (btest(outparam%mode,0) .and. trcl%ntracks>0) then
!output age points and interpolated age surface for each output timestep
writeages: do l=outparam%first,outparam%last,outparam%step
    !l2 is the index for step l in master array
    l2 = (l-outparam%first)/outparam%step+1

    !count number of tracks for this timestep
    write_tracks = 0
    do i=1,trcl%ntracks
        if (trcl%tracks(i,5)==l) then
            write_tracks = write_tracks+1
        end if
    end do

    !create output file age calculation results
    call int_to_char (cistep,4,l)
    open (unit=40,file=trim(outparam%dir)//'ages-points_t'//cistep//'.dat',status='replace')

    !write header
    write(40,'(a)') 'TITLE = "Thermochron Age Points"'
    write(40,'(a)') 'FILETYPE = FULL'

    anytracks1: if (write_tracks==0) then

    write (*,'(a,i4)') 'No surfaced tracks for step ', l

    !Write Coordinates and Variables
    write(40, '(a,i10,a)') 'VARIABLES = "X", "Y", "Z"'
    write(40, '(a,I8,a)') 'Zone T="Thermochron Age Points", I=1, DATAPACKING=BLOCK'
    write(40,'(a,ES12.5E2)') 'SOLUTIONTIME = ', trcl%time(l2)
    write(40,'(3ES12.5E2)') 0.d0, 0.d0, 0.d0 

    else anytracks1
    !write variable list dynamically
    ivar = 0
    nvar = 3 + outparam%nages + 3*outparam%origin + outparam%maxdepth + outparam%ageID
    write (40,'(a)', advance='no') 'VARIABLES = '
    call write_variable_list ('X', nvar, ivar, 40)
    call write_variable_list ('Y', nvar, ivar, 40)
    call write_variable_list ('Z', nvar, ivar, 40)
    if (outparam%Mad_AHe_std ==1) call write_variable_list ('Mad_AHe_std', nvar, ivar, 40)
    if (outparam%Mad_Ahe_small ==1) call write_variable_list ('Mad_Ahe_small', nvar, ivar, 40)
    if (outparam%Mad_Ahe_medium ==1) call write_variable_list ('Mad_Ahe_medium', nvar, ivar, 40)
    if (outparam%Mad_Ahe_large ==1) call write_variable_list ('Mad_Ahe_large', nvar, ivar, 40)
    if (outparam%Mad_Ahe_lowdam ==1) call write_variable_list ('Mad_Ahe_LowDam', nvar, ivar, 40)
    if (outparam%Mad_Ahe_moddam ==1) call write_variable_list ('Mad_Ahe_ModDam', nvar, ivar, 40)
    if (outparam%Mad_Ahe_hidam ==1) call write_variable_list ('Mad_Ahe_HighDam', nvar, ivar, 40)
    if (outparam%Mad_ZHe ==1) call write_variable_list ('Mad_ZHe', nvar, ivar, 40)
    if (outparam%Mad_ArKFs ==1) call write_variable_list ('Mad_Ar', nvar, ivar, 40)
    if (outparam%Mad_ArBio ==1) call write_variable_list ('Mad_ArBiotite', nvar, ivar, 40)
    if (outparam%Mad_ArHorn ==1) call write_variable_list ('Mad_ArHornblende', nvar, ivar, 40)
    if (outparam%Mad_ArMusc ==1) call write_variable_list ('Mad_ArMuscovite', nvar, ivar, 40)
    if (outparam%Mad_Ti_UTh_He ==1) call write_variable_list ('Mad_Ti_UTh_He', nvar, ivar, 40)
    if (outparam%Mad_AFT ==1) call write_variable_list ('Mad_AFT', nvar, ivar, 40)
    if (outparam%Mad_ZFT ==1) call write_variable_list ('Mad_ZFT', nvar, ivar, 40)
    if (outparam%Muscovite ==1) call write_variable_list ('Muscovite', nvar, ivar, 40)
    if (outparam%ZFT ==1) call write_variable_list ('ZFT', nvar, ivar, 40)
    if (outparam%Apatite_U_Th_Pb ==1) call write_variable_list ('Apatite_U_Th_Pb', nvar, ivar, 40)
    if (outparam%Biotite ==1) call write_variable_list ('Biotite', nvar, ivar, 40)
    if (outparam%Rutile_U_Pb ==1) call write_variable_list ('Rutile_U_Pb', nvar, ivar, 40)
    if (outparam%Titanite_U_Pb ==1) call write_variable_list ('Titanite_U_Pb', nvar, ivar, 40)
    if (outparam%Zircon_U_Pb ==1) call write_variable_list ('Zircon_U_Pb', nvar, ivar, 40)
    if (outparam%origin==1) then
        call write_variable_list ('xorigin', nvar, ivar, 40)
        call write_variable_list ('yorigin', nvar, ivar, 40)
        call write_variable_list ('zorigin', nvar, ivar, 40)
    end if
    if (outparam%maxdepth==1) call write_variable_list ('max_depth', nvar, ivar, 40)
    if (outparam%ageID==1) call write_variable_list ('trackID', nvar, ivar, 40)

    !Write Coordinates and Variables
    write (40, '(a,I8,a)') 'Zone T="Thermochron Age Points", I=', write_tracks, ', DATAPACKING=BLOCK'
    write(40,'(a,ES12.5E2)') 'SOLUTIONTIME = ', trcl%time(l2)
    charcnt = 0
    do i=1,trcl%ntracks
    !go through all tracks
        if (trcl%tracks(i,5)==l) then
        !if track surfaces at this timestep: write last point of track (at surface)
        write(40,'(ES13.5E2)', advance='no') trcl%x(trcl%tracks(i,1),trcl%tracks(i,3))
        charcnt = charcnt + 13
        if (charcnt > 160) then
            write(40,'(a)') ''
            charcnt = 0
        endif
        endif
    enddo
    write(40,'(a)') ''
    charcnt = 0
    do i=1,trcl%ntracks
    !go through all tracks
        if (trcl%tracks(i,5)==l) then
        !if track surfaces at this timestep: write last point of track (at surface)
        write(40,'(ES13.5E2)', advance='no') trcl%y(trcl%tracks(i,1),trcl%tracks(i,3))
        charcnt = charcnt + 13
        if (charcnt > 160) then
            write(40,'(a)') ''
            charcnt = 0
        endif
        end if
    enddo
    write(40,'(a)') ''
    charcnt = 0
    do i=1,trcl%ntracks
    !go through all tracks
        if (trcl%tracks(i,5)==l) then
        !if track surfaces at this timestep: write last point of track (at surface)
        write(40,'(ES13.5E2)', advance='no') trcl%z(trcl%tracks(i,1),trcl%tracks(i,3))
        charcnt = charcnt + 13
        if (charcnt > 160) then
            write(40,'(a)') ''
            charcnt = 0
        endif
        endif
    enddo
    write(40,'(a)') ''

    !write ages for all points
    if (outparam%Mad_AHe_std ==1) call write_ages_points (trcl%Mad_AHe_std, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_Ahe_small ==1) call write_ages_points (trcl%Mad_Ahe_small, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_Ahe_medium ==1) call write_ages_points (trcl%Mad_Ahe_medium, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_Ahe_large ==1) call write_ages_points (trcl%Mad_Ahe_large, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_Ahe_lowdam ==1) call write_ages_points (trcl%Mad_Ahe_lowdam, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_Ahe_moddam ==1) call write_ages_points (trcl%Mad_Ahe_moddam, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_Ahe_hidam ==1) call write_ages_points (trcl%Mad_Ahe_hidam, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_ZHe ==1) call write_ages_points (trcl%Mad_ZHe, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_ArKFs ==1) call write_ages_points (trcl%Mad_ArKFs, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_ArBio ==1) call write_ages_points (trcl%Mad_ArBio, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_ArHorn ==1) call write_ages_points (trcl%Mad_ArHorn, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_ArMusc ==1) call write_ages_points (trcl%Mad_ArMusc, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_Ti_UTh_He ==1) call write_ages_points (trcl%Mad_Ti_UTh_He, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_AFT ==1) call write_ages_points (trcl%Mad_AFT, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Mad_ZFT ==1) call write_ages_points (trcl%Mad_ZFT, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Muscovite ==1) call write_ages_points (trcl%Muscovite, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%ZFT ==1) call write_ages_points (trcl%ZFT, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Apatite_U_Th_Pb ==1) call write_ages_points (trcl%Apatite_U_Th_Pb, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Biotite ==1) call write_ages_points (trcl%Biotite, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Rutile_U_Pb ==1) call write_ages_points (trcl%Rutile_U_Pb, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Titanite_U_Pb ==1) call write_ages_points (trcl%Titanite_U_Pb, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%Zircon_U_Pb ==1) call write_ages_points (trcl%Zircon_U_Pb, &
        trcl%ntracks, trcl%tracks(:,5), l, 40)
    if (outparam%origin==1) then
        charcnt = 0
        do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
            write(40,'(ES13.5E2)', advance='no') trcl%x(trcl%tracks(i,1),trcl%tracks(i,2))
            charcnt = charcnt + 13
            if (charcnt > 160) then
                write(40,'(a)') ''
                charcnt = 0
            endif
            endif
        enddo
        write(40,'(a)') ''
        charcnt = 0
        do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
            write(40,'(ES13.5E2)', advance='no') trcl%y(trcl%tracks(i,1),trcl%tracks(i,2))
            charcnt = charcnt + 13
            if (charcnt > 160) then
                write(40,'(a)') ''
                charcnt = 0
            endif
            endif
        enddo
        write(40,'(a)') ''
        charcnt = 0
        do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
            write(40,'(ES13.5E2)', advance='no') trcl%z(trcl%tracks(i,1),trcl%tracks(i,2))
            charcnt = charcnt + 13
            if (charcnt > 160) then
                write(40,'(a)') ''
                charcnt = 0
            endif
            endif
        enddo
        write(40,'(a)') ''
    end if
    if (outparam%maxdepth==1) then
        write (40,'(a)') 'SCALARS max_depth FLOAT 1'
        write (40,'(a)') 'LOOKUP_TABLE default'
        charcnt = 0
        do i=1,trcl%ntracks
            if (trcl%tracks(i,5)==l) then
                !Get minimum Z value of track
                maxdepth = trcl%z(trcl%tracks(i,1),trcl%tracks(i,2))
                do j = trcl%tracks(i,2), trcl%tracks(i,3)
                    maxdepth = min(maxdepth, trcl%z(trcl%tracks(i,1), j))
                end do
                write(40,'(ES13.5E2)', advance='no') maxdepth
                charcnt = charcnt + 13
                if (charcnt > 160) then
                    write(40,'(a)') ''
                    charcnt = 0
                endif
            end if
        end do
    end if
    if (outparam%ageID==1) call write_ages_points (1.0*trcl%tracks(:,7), &
        trcl%ntracks, trcl%tracks(:,5), l, 40)

    
    end if anytracks1
    close (40) 

    !write suface with interpolated ages
    open (unit=41,file=trim(outparam%dir)//'ages-surface_t'//cistep//'.dat',status='replace')
    !write header
    write(41,'(a)') 'TITLE = "Thermochron Age Surface"'
    write(41,'(a)') 'FILETYPE = FULL'


    anytracks2: if (write_tracks < 4) then

    write(41, '(a,i10,a)') 'VARIABLES = "X", "Y", "Z"'
    write(41, '(a,I8,a)') 'Zone T="Thermochron Age Surface", I=1, DATAPACKING=BLOCK'
    write(41,'(a,ES12.5E2)') 'SOLUTIONTIME = ', trcl%time(l2)
    write(41,'(3ES12.5E2)') 0.d0, 0.d0, 0.d0 

    else anytracks2

    ivar = 0
    nvar = 3 + outparam%nages + 3*outparam%origin + outparam%maxdepth 
    write (41,'(a)', advance='no') 'VARIABLES = '
    call write_variable_list ('X', nvar, ivar, 41)
    call write_variable_list ('Y', nvar, ivar, 41)
    call write_variable_list ('Z', nvar, ivar, 41)
    if (outparam%Mad_AHe_std ==1) call write_variable_list ('Mad_AHe_std', nvar, ivar, 41)
    if (outparam%Mad_Ahe_small ==1) call write_variable_list ('Mad_Ahe_small', nvar, ivar, 41)
    if (outparam%Mad_Ahe_medium ==1) call write_variable_list ('Mad_Ahe_medium', nvar, ivar, 41)
    if (outparam%Mad_Ahe_large ==1) call write_variable_list ('Mad_Ahe_large', nvar, ivar, 41)
    if (outparam%Mad_Ahe_lowdam ==1) call write_variable_list ('Mad_Ahe_LowDam', nvar, ivar, 41)
    if (outparam%Mad_Ahe_moddam ==1) call write_variable_list ('Mad_Ahe_ModDam', nvar, ivar, 41)
    if (outparam%Mad_Ahe_hidam ==1) call write_variable_list ('Mad_Ahe_HighDam', nvar, ivar, 41)
    if (outparam%Mad_ZHe ==1) call write_variable_list ('Mad_ZHe', nvar, ivar, 41)
    if (outparam%Mad_ArKFs ==1) call write_variable_list ('Mad_Ar', nvar, ivar, 41)
    if (outparam%Mad_ArBio ==1) call write_variable_list ('Mad_ArBiotite', nvar, ivar, 41)
    if (outparam%Mad_ArHorn ==1) call write_variable_list ('Mad_ArHornblende', nvar, ivar, 41)
    if (outparam%Mad_ArMusc ==1) call write_variable_list ('Mad_ArMuscovite', nvar, ivar, 41)
    if (outparam%Mad_Ti_UTh_He ==1) call write_variable_list ('Mad_Ti_UTh_He', nvar, ivar, 41)
    if (outparam%Mad_AFT ==1) call write_variable_list ('Mad_AFT', nvar, ivar, 41)
    if (outparam%Mad_ZFT ==1) call write_variable_list ('Mad_ZFT', nvar, ivar, 41)
    if (outparam%Muscovite ==1) call write_variable_list ('Muscovite', nvar, ivar, 41)
    if (outparam%ZFT ==1) call write_variable_list ('ZFT', nvar, ivar, 41)
    if (outparam%Apatite_U_Th_Pb ==1) call write_variable_list ('Apatite_U_Th_Pb', nvar, ivar, 41)
    if (outparam%Biotite ==1) call write_variable_list ('Biotite', nvar, ivar, 41)
    if (outparam%Rutile_U_Pb ==1) call write_variable_list ('Rutile_U_Pb', nvar, ivar, 41)
    if (outparam%Titanite_U_Pb ==1) call write_variable_list ('Titanite_U_Pb', nvar, ivar, 41)
    if (outparam%Zircon_U_Pb ==1) call write_variable_list ('Zircon_U_Pb', nvar, ivar, 41)
    if (outparam%origin==1) then
        call write_variable_list ('xorigin', nvar, ivar, 40)
        call write_variable_list ('yorigin', nvar, ivar, 40)
        call write_variable_list ('zorigin', nvar, ivar, 40)
    end if
    if (outparam%maxdepth==1) call write_variable_list ('max_depth', nvar, ivar, 40)

    write(41, '(a29,i10,a11,i10,a)') 'ZONE T="Ages Surface", NODES=',surface(l2)%nsurface, &
          ', ELEMENTS=',surface(l2)%nt,', DATAPACKING=BLOCK, ZONETYPE=FETRIANGLE'
    write(41,'(a,ES12.5E2)') 'SOLUTIONTIME = ', trcl%time(l2)

    call write_coord_surface (surface(l2)%x, surface(l2)%nsurface, 41)
    call write_coord_surface (surface(l2)%y, surface(l2)%nsurface, 41)
    call write_coord_surface (surface(l2)%z, surface(l2)%nsurface, 41)

    !write ages for all points
    do k = 1, nvar
        call write_var_surface (surface(l2)%ages(:,k), surface(l2)%nsurface, 41)
    end do
    ! Write node-element connectivity
    do i=1,surface(l2)%nt
        write(41,'(3I10)') surface(l2)%icon(1:3,i)
    enddo
    end if anytracks2
    close(41)

end do writeages
end if
    
if (btest(outparam%mode,1) .or. btest(outparam%mode,2)) then
    write (*,*) 'ERROR: Output of tracks only enabled for VTK output'
end if

end subroutine write_post_track_tecplot

subroutine write_variable_list (varname, nvars, ivar, iunit)
!This subroutine writes the variable list at the beginning of a Tecplot file

implicit none

character*(*) :: varname
integer, intent(in) :: nvars, iunit
integer, intent(inout) :: ivar

ivar = ivar+1
write (iunit,'(a)', advance='no') '"'//varname
if (ivar < nvars) then
    write (iunit,'(a)', advance='no') '", '
else
    write (iunit,'(a)') '"'
end if

end subroutine write_variable_list

!------------------------------------------------------------------------------|

subroutine write_ages_points (ages, ntracks, tracks, step, iunit)

implicit none

real, dimension(:), intent(in) :: ages

integer, dimension(:), intent(in) :: tracks
integer, intent(in) :: ntracks, step, iunit


integer i, charcnt

charcnt = 0
do i=1,ntracks
!go through all tracks
    if (tracks(i)==step) then
    !if track surfaces at this timestep: write last point of track (at surface)
    write(iunit,'(ES13.5E2)', advance='no') ages(i)
    charcnt = charcnt + 13
    if (charcnt > 160) then
        write(iunit,'(a)') ''
        charcnt = 0
    endif
    endif
enddo
if (charcnt > 0) write(iunit,'(a)') ''

end subroutine write_ages_points

subroutine write_coord_surface (var, nsurface, iunit)

implicit none

double precision, dimension(:), intent(in) :: var
integer, intent(in) :: nsurface, iunit

integer i, charcnt

charcnt = 0
do i=1,nsurface
!go through all tracks
    write(iunit,'(ES13.5E2)', advance='no') var(i)
    charcnt = charcnt + 13
    if (charcnt > 160) then
        write(iunit,'(a)') ''
        charcnt = 0
    endif
enddo
if (charcnt > 0) write(iunit,'(a)') ''

end subroutine write_coord_surface


subroutine write_var_surface (var, nsurface, iunit)

implicit none

real, dimension(:), intent(in) :: var
integer, intent(in) :: nsurface, iunit

integer i, charcnt

charcnt = 0
do i=1,nsurface
!go through all tracks
    write(iunit,'(ES13.5E2)', advance='no') var(i)
    charcnt = charcnt + 13
    if (charcnt > 160) then
        write(iunit,'(a)') ''
        charcnt = 0
    endif
enddo
if (charcnt > 0) write(iunit,'(a)') ''

end subroutine write_var_surface

end module m_write_tecplot

