module m_orography
    contains

subroutine orography (elev, precip, f_nx, f_ny, dx, dy, params)

! not tested, not vectorized, in development

use definitions

implicit none

double precision, dimension(:,:) :: elev, precip
double precision :: dx, dy
integer:: f_nx, f_ny
type (parameters) params

double precision, dimension(:,:), allocatable :: h,pr,w
complex*16, dimension(:,:), allocatable :: work
complex*16, dimension(:), allocatable :: d

integer noro,i,j
double precision cw,u,v,Nm,tauf,taus,hw,pm,sigma
double precision k,l,mkl,Nmsigma
complex*16 im

!im is imaginary unit
im=cmplx(0.d0,1.d0)

pm = params%precipitation/(365.25d0 * 24.d0 * 3.6d3) !pm is mean precipitation in m/s
u = params%precipitation_parameters(1) !u is E-W wind velocity
v = params%precipitation_parameters(2) !v is S-N wind velocity
cw = params%precipitation_parameters(3) !cw is uplift sensitivity factor (0.001-0.02 kg/m3)
Nm = params%precipitation_parameters(4) !Nm is moist stability frequency (0-0.01 1/s)
hw = params%precipitation_parameters(5) !hw is Water vapour scale height (1-5 km)
taus = params%precipitation_parameters(6) !taus is conversion time (200-2000 s)
tauf = params%precipitation_parameters(7) !tauf is fallout time (200-2000 s)
noro = int(params%precipitation_parameters(8)) !noro is the number of gridnodes for FFT calucation

! grid dimension
allocate (h(noro,noro),pr(noro,noro))
allocate (w(f_nx,f_ny))

w = elev
call InterpolSmooth (w,f_nx,f_ny,h,noro,noro)
deallocate (w)

allocate (work(noro,noro))

! FFT

allocate (d(noro))
  do j=1,noro
    do i=1,noro
    d(i)=dcmplx(h(i,j),0.d0)
    enddo
  call four1 (d,noro,-1)
    do i=1,noro
    work(j,i)=d(i)
    enddo
  enddo
deallocate (d)

allocate (d(noro))
  do i=1,noro
    do j=1,noro
    d(j)=work(j,i)
    enddo
  call four1 (d,noro,-1)
    do j=1,noro
    work(j,i)=d(j)
    enddo
  enddo 
deallocate (d)

  do j=1,noro
  do i=1,noro
  work(j,i)=work(j,i)/noro/noro
  enddo
  enddo

!Orographic filter

  do j=1,noro
   do i=1,noro
   k=float(i-1)/dx
   if (i.gt.noro/2) k=-float(noro-i+1)/dx
   l=float(j-1)/dy
   if (j.gt.noro/2) l=-float(noro-j+1)/dy
   sigma=u*k+v*l
   mkl=0.d0
   Nmsigma=Nm**2-sigma**2
   if (abs(sigma).gt.tiny(sigma).and.Nmsigma.gt.tiny(Nmsigma)) mkl=sqrt((Nmsigma/sigma**2)*(k**2+l**2))
   work(j,i)=Cw*im*sigma*work(j,i)/(1.d0-im*mkl*Hw)/(1.d0+im*sigma*taus)/(1.d0+im*sigma*tauf)
   enddo
 enddo

!Inverse FFT

allocate (d(noro))
  do i=1,noro
    do j=1,noro
    d(j)=work(j,i)
    enddo
  call four1 (d,noro,1)
    do j=1,noro
    work(j,i)=d(j)
    enddo
  enddo
deallocate (d)

allocate (d(noro))
  do j=1,noro
    do i=1,noro
    d(i)=work(j,i)
    enddo
  call four1 (d,noro,1)
    do i=1,noro
    pr(i,j)=real(d(i))
    enddo
  enddo
deallocate (d)

deallocate (work)

pr=pr*3600.d0*24.d0*365.25d0/1.d3

pr=pr+pm
pr=max(pr,pm)

allocate (w(f_nx,f_ny))
call InterpolLinear (pr,noro,noro,w,f_nx,f_ny)
precip = w * dx * dy

deallocate (w,h,pr)

end subroutine orography


subroutine orography_simple (elev, precip, f_nx, f_ny, dx, dy, params)

! not tested, not vectorized, in development

use definitions

implicit none

include 'mpif.h'

double precision, dimension(:,:) :: elev, precip
double precision :: dx, dy
integer:: f_nx, f_ny
type (parameters) params

integer :: iproc, nproc, ierr
integer :: i,j, jmin, jmax, nwindow
integer, dimension(:), allocatable :: ipeak

double precision :: p0, p1, zpeakj, ipeakj, plee
double precision :: pluvbase, pmax, pmin, zpmax, dxluv, dxlee, smooth
double precision, dimension(:), allocatable :: zpeak

!------------------------------------------------------------------------------!
!------------------------------------------------------------------------------!

call mpi_comm_size (mpi_comm_world,nproc,ierr)
call mpi_comm_rank (mpi_comm_world,iproc,ierr)

pluvbase = params%precipitation
pmax = params%precipitation_parameters(1) !pmax is maximum precpiation achievable
pmin = params%precipitation_parameters(2) !pmin is minimum precipitation, lee baselevel
zpmax = params%precipitation_parameters(3) !zpmax is elevation above which p is pmax
dxluv = params%precipitation_parameters(4) !dxlux is characteristic length at luv side
dxlee = params%precipitation_parameters(5) !dxlee is characteristic length at lee side
smooth = params%precipitation_parameters(6) !smooth is smoothing window size

!for each line along x, find max elevation and peak position
allocate(zpeak(f_ny), ipeak(f_ny))
do j = 1,f_ny
    !peak elevation and precipitation
    zpeak(j) = maxval(elev(:,j),DIM=1) 
    ipeak(j) = maxloc(elev(:,j),DIM=1) 
end do

nwindow = idint(smooth * 1.0d3 / dy * 0.5d0) 
do j = 1,f_ny
    !smooth ipeak and zpeak
    jmin = max(1, j - nwindow)
    jmax = min(f_ny, j + nwindow)
    zpeakj = sum(zpeak(jmin:jmax)) / (jmax-jmin+1)
    ipeakj = sum(ipeak(jmin:jmax))*1.d0 / (jmax-jmin+1)

    p0 = zpeakj / zpmax * (pmax - pluvbase)
    !scale down lee baselevel by same factor of peak increase compared to luv
    plee = pluvbase**2 / (p0 + pluvbase)
    p1 = p0 - plee
    do i = 1,idint(ipeakj)
        precip(i,j) = pluvbase + p0 * exp( (i - ipeakj) * dx * 1.0d-3 / dxluv)
    end do
    do i = idint(ipeakj)+1, f_nx
        !precipitation at lee side is at least pmin
        precip(i,j) = max(pmin, plee + p1 * exp( (ipeakj -i) * dx * 1.0d-3 / dxlee))
    end do
end do

end subroutine orography_simple

subroutine InterpolLinear(a,nxa,nya,b,nx,ny)
! Linear interpolation (less smooth but faster)
! interpolates regular grid a of size nxa*nya smoothly onto grid b of size nx*ny 
! and same real-life dimensions, i.e. other resolution

integer nxa,nya,nx,ny
double precision, dimension(nxa,nya), intent(in) :: a
double precision, dimension(nx,ny), intent(out) :: b

integer i,j,ia,ja
double precision r,s

do j=1,ny
  ja=1+((nya-1)*(j-1))/(ny-1)
  s=(float(j-1)/(ny-1)-float(ja-1)/(nya-1))/(nya-1)*2.d0-1.d0
    if (ja.eq.nya) then
    ja=nya-1
    s=1.d0
    endif
    do i=1,nx
        ia=1+((nxa-1)*(i-1))/(nx-1)
        r=(float(i-1)/(nx-1)-float(ia-1)/(nxa-1))/(nxa-1)*2.d0-1.d0
        if (ia.eq.nxa) then
            ia=nxa-1
            r=1.d0
        endif
        b(i,j)=(a(ia,ja)*(1.d0-r)*(1.d0-s) &
              +a(ia+1,ja)*(1.d0+r)*(1.d0-s) &
              +a(ia+1,ja+1)*(1.d0+r)*(1.d0+s) &
              +a(ia,ja+1)*(1.d0-r)*(1.d0+s))/4.d0
    enddo
enddo

end subroutine InterpolLinear

subroutine InterpolSmooth(a,nxa,nya,b,nx,ny)
! Smoothing interpolation (less fast but smoother)
! interpolates regular grid a of size nxa*nya smoothly onto grid b of size nx*ny 
! and same real-life dimensions, i.e. other resolution

integer nxa,nya,nx,ny
double precision, dimension(nxa,nya), intent(in) :: a
double precision, dimension(nx,ny), intent(out) :: b

integer i,j,ia,ja,ib,jb

do j=1,ny
    ja=1+((nya-1)*(j-1))/(ny-1)
    if (ja.eq.nya) ja=nya-1
    jb=1+((nya-1)*j)/(ny-1)
    if (jb.gt.nya) jb=nya
    do i=1,nx
        ia=1+((nxa-1)*(i-1))/(nx-1)
        if (ia.eq.nxa) ia=nxa-1
        ib=1+((nxa-1)*i)/(nx-1)
        if (ib.gt.nxa) ib=nxa
        b(i,j)=sum(a(ia:ib,ja:jb))/size(a(ia:ib,ja:jb))
    enddo
enddo

end subroutine InterpolSmooth

end module m_orography
